Apaq Framework
==============

Apaq's framework that puts in the missing links in the Java domain. Currently it consists of 2 major parts namely Repository and Criteria. Both are described further down this page.


- [Criteria](#criteria) - _used to be filter4j_
- [Repository](#repository) - _used to be crud4j_
- [Authors](#authors)

<a name="criteria"></a>
Criteria
========

A simple and general way of filtering data with built in implementation for <b>Collections</b>, <b>JPA</b> and <b>MongoDb</b>. It makes it possible to use the <i>same Criteria with <u>several types of backends</u></i>.

## Contents

- [Example 1 - filtering data from a collection](#criteria-example1)
- [Example 2 - filtering data from JPA](#criteria-example2)
- [Example 3 - filtering data from MongoDb](#criteria-example3)
- [Usage](#criteria-usage)

<a name="criteria-example1"></a>
## Example 1 - filtering data from a collection

Lets say we create a class that contains data about a person:
```java
public static class Person {

    public Person(String name, int age, boolean extraTerrestrial, double weightInKilos) {
        this.name = name;
        this.age = age;
        this.extraTerrestrial = extraTerrestrial;
        this.weightInKilos = weightInKilos;
    }
    
    private String name;
    private int age;
    private boolean extraTerrestrial;
    private double weightInKilos;

    public int getAge() {
        return age;
    }

    public boolean getExtraTerrestrial() {
        return extraTerrestrial;
    }

    public String getName() {
        return name;
    }

    public double getWeightInKilos() {
        return weightInKilos;
    }
    
    @Override
    public String toString() {
        return name + " [age="+age+";et="+extraTerrestrial+";weight="+weightInKilos+"kg]";
    }
    
}
```

... and that we create a list of totally random persons:
```java
List<Person> persons = new ArrayList<Person>();
persons.add(new Person("Michael Krog", 33, false, 104.3));
persons.add(new Person("Michael J. Fox", 50, false, 50.2));
persons.add(new Person("Michael Jackson", 50, true, 55.3));
persons.add(new Person("Luke Skywalker", 25, true, 67.4));        
```

Then we have som persons with some properties which we can use for filtering.
We start by creating af filter that find a person whose age is less than 35, name
starts with 'Michael', weight is greater than 55kg and who is not an extraTerrestrial:
```java
Criteria criteria = new Criteria(Rules.and(
  Rules.lessThan("age", 35),
  Rules.like("name", "Michael*"),
  Rules.greaterThan("weightInKilos", 55),
  Rules.equals("extraTerrestrial", false)
  )
);      
```

Ok. So now we have some data in a collection and a criteria. How do we get the matching results from
the collection? Like this:
```java
List<Person> result = CriteriaUtil.filterCollection(persons, criteria);

// Print out the results
for(Person person : result) {
  System.out.println("This one is a match: " + person);
}
```
This would output: "This one is a match: Michael Krog [age=33;et=false;weight=104.3kg]"

<a name="criteria-example2"></a>
## Example 2 - filtering data from JPA
```java
EntityManager em = ....;
Query q = CriteriaForJpa.translate(em, Person.class, criteria);
```

<a name="criteria-example3"></a>
## Example 3 - filtering data from MongoDb
```java
Query q = CriteriaForMongo.translate(Person.class, criteria);
```

<a name="usage"></a>
## Usage

### via Maven

Dependency definitions to your `pom.xml`.

```xml
<dependency>
  <groupId>dk.apaq.framework</groupId>
  <artifactId>criteria-api</artifactId>
  <version>1.0.2</version>
</dependency>
```

<a name="repository"></a>
Repository
======

A generel interface for data repositories with built in implementation for JPA, MongoDB and collections. Supports filtering via the Criteria API. 


## Contents

- [Example](#repository-example)
- [Usage](#repository-usage)

<a name="repository-example"></a>
## Example

Lets say we have a class named Planet that looks like this:
```java
public class Planet {
    
    private String name;
    private long population;

    public Planet(String name, long population) {
        this.name = name;
        this.population = population;
    }

    public String getName() {
        return name;
    }

    public long getPopulation() {
        return population;
    }
 
}
```

We can create a list of Planet instances and use that as a crud, like this:
```java
// Create a list of planets
List<Planet> planets = Arrays.asList(new Planet[]{
    new Planet("Earth", 7000000000L), new Planet("Pluto", 0)
});

//Create crud from list of planets
Repository<String, Planet> rep = new CollectionRepository<Planet>(planets);

//Get id of first planet and read it from the crud
String id = rep.findAllIds().get(0);
Planet p = rep.findOne(id);

//Or retrieve list of all planets and get it from that
p = rep.findAll().get(0);

//Filter the result
Criteria criteria = new Criteria(Rules.like("name", "ea*"), new Sorter("name"));
List<Planet> result = rep.findAll(criteria);
System.out.println("No of matches: " + result.size());
```

Now, lets say that we just for the fun of it want to have our planets stored in MongoDb, a JPA supported database and a Collection. Lets create a repository for each, save a planet to each and then search in all 3 repositories by the same criteria.

```java
// Create Mongo Repository
MongoTemplate mongoTemplate = ....;
Repository<String, Planet> mongoRep = new MongoDbRepository(mongotTemplate, Planet.class);

// Create JPA Repository
EntityManager em = ....;
Repository<String, Planet> jpaRep = new EnætityManagerRepository(em, Planet.class);

//Create Collection Repository
Repository<String, Planet> collectionRep = new CollectionRepository<Planet>(planets);

//Create a new planet in each repository
mongoRep.save(new Planet("Mars", 13123123L));
jpaRep.save(new Planet("Venus", 1233443L));
collectionRep.save(new Planet("Earth", 41313122L));

//Filter the results
Criteria criteria = new Criteria(Rules.equals("name", "Mars"), new Sorter("name"));
List<Planet> mongoResult = mongoRep.findAll(criteria);
List<Planet> jpaResult = jpaRep.findAll(criteria);
List<Planet> collectionResult = collectionRep.findAll(criteria);
```

<a name="repository-usage"></a>
## Usage

### via Maven

Dependency definitions to your `pom.xml`.

```xml
<dependency>
  <groupId>dk.apaq.framework</groupId>
  <artifactId>repository</artifactId>
  <version>1.0.2</version>
</dependency>
```


<a name="authors"></a>
## Authors

- Michael Krog / [michaelkrog](https://github.com/michaelkrog)