package dk.apaq.framework.common.beans.finance;

import java.io.Serializable;
import java.util.regex.Pattern;

import org.apache.commons.lang.Validate;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasypt.encryption.StringEncryptor;

/**
 * Defines a Payment Card.
 */
public class Card implements Serializable, Cloneable {

    private static final int TEN = 10;
    public static final Pattern PATTERN_CARDNUMBER = Pattern.compile("[0-9]{12,16}");
    public static final Pattern PATTERN_CVD = Pattern.compile("[0-9]{3,4}");
    private String cardHolder;
    private String cardNumber;
    private String cardNumberEncrypted;
    private int expireYear;
    private int expireMonth;
    private boolean valid;
    private String cvd;
    private PaymentInstrument paymentInstrument;

    protected Card() { /* LEFT EMPTY FOR USE BY FX: JPA */ }

    public Card(Card card) {
        this.cardHolder = card.cardHolder;
        this.cardNumber = card.cardNumber;
        this.cardNumberEncrypted = card.cardNumberEncrypted;
        this.cvd = card.cvd;
        this.expireMonth = card.getExpireMonth();
        this.expireYear = card.getExpireYear();
        this.valid = card.valid;
        this.paymentInstrument = card.paymentInstrument;
    }

    /**
     * Constructs a new instance of Card.
     *
     * @param cardHolder The name of the Card holder.
     * @param cardNumber Thee card number.
     * @param expireYear The year of expiration fx. 2012
     * @param expireMonth The month of expiration (1=January, 2=February etc.)
     * @param cvd The CVD(security code) from the Card.
     */
    public Card(String cardHolder, String cardNumber, int expireYear, int expireMonth, String cvd, StringEncryptor encryptor) {
        Validate.notNull(encryptor, "encryptor is null.");
        Validate.notNull(cardNumber, "cardNumber is null.");
        Validate.notNull(cvd, "cvd is null.");

        //Remove any spaces in card number.
        cardNumber = cardNumber.replace(" ", "");
        Validate.isTrue(PATTERN_CARDNUMBER.matcher(cardNumber).matches(), "Cardnumber must be numeric and 12-16 digits.");

        //if expireYear less than 100 then perhaps it was given as year within current century fx 16 instead of 2016. Add 2000 years.
        if (expireYear < 100) {
            expireYear += 2000;
        }

        Validate.isTrue(expireYear >= 2000 && expireYear < 2100, "expireYear must be between 2000 and 2100.");
        Validate.isTrue(expireMonth >= 1 && expireMonth <= 12, "expireMonth must be between 1 and 12(both included).");

        this.cardHolder = cardHolder;
        this.cardNumber = obfuscateNumber(cardNumber);
        this.cardNumberEncrypted = encryptor.encrypt(cardNumber);
        this.valid = isCardNumberValid(cardNumber);
        this.expireYear = expireYear;
        this.expireMonth = expireMonth;
        this.cvd = cvd;
        this.paymentInstrument = resolveInstrument(cardNumber);
    }

    /**
     * Constructs a new instance of Card without the name of the Card Holder.
     *
     * @param cardNumber Thee card number.
     * @param expireYear The year of expiration fx. 2012
     * @param expireMonth The month of expiration (1=January, 2=February etc.)
     * @param cvd The CVD(security code) from the Card.
     */
    public Card(String cardNumber, int expireYear, int expireMonth, String cvd, StringEncryptor encryptor) {
        this(null, cardNumber, expireYear, expireMonth, cvd, encryptor);
    }

    /**
     * Retrieves the name of the card holder.
     *
     * @return The name
     */
    public String getCardHolder() {
        return cardHolder;
    }

    /**
     * Retrieves the obfuscated card number. This card number it suitable for displaying fx. for receipts. It will display only the last for
     * chars, fx. **** **** **** 1234
     *
     * @return The obfuscated card number
     */
    public String getCardNumber() {
        return cardNumber;
    }

    /**
     * Retrieves the original card number. The method will decrypt the card number.
     *
     * @return The original card number
     */
    public String getCardNumber(StringEncryptor encryptor) {
        if (cardNumberEncrypted == null) {
            return null;
        } else {
            return encryptor.decrypt(cardNumberEncrypted);
        }
    }

    /**
     * Retrieves the CVD.
     *
     * @return The cvd.
     */
    public String getCvd(StringEncryptor encryptor) {
        return cvd;
    }

    /**
     * Retrieves the expiration month. It will be the number of month in year for expiration.
     *
     * @return The month.
     */
    public int getExpireMonth() {
        return expireMonth;
    }

    /**
     * Retrieves the expiration year.
     *
     * @return The year.
     */
    public int getExpireYear() {
        return expireYear;
    }

    /**
     * Whether the cardNumber is a valid card number by doing a Luhn check.
     *
     * @return True/False
     */
    public boolean isValid() {
        return valid;
        /*int sum = 0;
         boolean alternate = false;
         for (int i = cardNumber.length() - 1; i >= 0; i--) {
         int n = Integer.parseInt(cardNumber.substring(i, i + 1));
         if (alternate) {
         n *= 2;
         if (n > TEN - 1) {
         n = (n % TEN) + 1;
         }
         }
         sum += n;
         alternate = !alternate;
         }
         return (sum % TEN == 0);*/
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Card) {
            Card other = (Card) obj;
            return new EqualsBuilder().
                    append(this.cardHolder, other.cardHolder).
                    append(this.cardNumber, other.cardNumber).
                    append(this.cvd, other.cvd).
                    append(this.expireMonth, other.expireMonth).
                    append(this.expireYear, other.expireYear).isEquals();
        }
        return false;
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder().
                append(this.cardHolder).
                append(this.cardNumber).
                append(this.cvd).
                append(this.expireMonth).
                append(this.expireYear).hashCode();

    }

    @Override
    public String toString() {
        return ToStringBuilder.reflectionToString(this);
    }

    public PaymentInstrument getPaymentInstrument() {
        return paymentInstrument;
    }

    /**
     * Retrieves the Payment Instrument detected.
     *
     * @return The payment instrument
     */
    private PaymentInstrument resolveInstrument(String number) {
        if (number == null || !isValid()) {
            return PaymentInstrument.Unknown;
        }

        if (isDankort(number)) {
            return PaymentInstrument.Dankort;
        }

        if (isVisaElectron(number)) {
            return PaymentInstrument.Visa_Electron;
        }

        if (isVisa(number)) {
            return PaymentInstrument.Visa;
        }

        if (isAmericanExpress(number)) {
            return PaymentInstrument.American_Express;
        }

        if (isDiners(number)) {
            return PaymentInstrument.Diners;
        }

        if (isJcb(number)) {
            return PaymentInstrument.Jcb;
        }

        if (isMasterCard(number)) {
            return PaymentInstrument.Mastercard;
        }

        return PaymentInstrument.Unknown;
    }

    private boolean isAmericanExpress(String number) {
        return number.startsWith("34") || number.startsWith("37");
    }

    private boolean isDankort(String number) {
        return number.startsWith("5019") || number.startsWith("4571");
    }

    private boolean isDiners(String number) {
        boolean value = number.startsWith("36")
                || number.startsWith("54")
                || number.startsWith("55")
                || number.startsWith("2014")
                || number.startsWith("2149");

        if (value != true && number.length() > 2) {
            int first3 = Integer.parseInt(number.substring(0, 2));
            value = first3 >= 300 && first3 <= 305;
        }
        return value;
    }

    private boolean isJcb(String number) {
        boolean value = false;

        if (number.length() > 3) {
            int first4 = Integer.parseInt(number.substring(0, 3));
            value = first4 >= 3528 && first4 <= 3589;
        }
        return value;
    }

    private boolean isMasterCard(String number) {
        boolean value = false;

        if (number.length() > 1) {
            int first2 = Integer.parseInt(number.substring(0, 2));
            value = first2 >= 51 && first2 <= 55;
        }
        return value;
    }

    private boolean isVisa(String number) {
        return number.startsWith("4");
    }

    private boolean isVisaElectron(String number) {
        boolean value = number.startsWith("4026")
                || number.startsWith("417500")
                || number.startsWith("4508")
                || number.startsWith("4844")
                || number.startsWith("4913")
                || number.startsWith("4917");

        if (value != true && number.length() > 2) {
            int first3 = Integer.parseInt(number.substring(0, 2));
            value = first3 >= 300 && first3 <= 305;
        }
        return value;
    }

    private boolean isCardNumberValid(String number) {
        assert number != null;
        int sum = 0;
        boolean alternate = false;
        for (int i = number.length() - 1; i >= 0; i--) {
            int n = Integer.parseInt(number.substring(i, i + 1));
            if (alternate) {
                n *= 2;
                if (n > TEN - 1) {
                    n = (n % TEN) + 1;
                }
            }
            sum += n;
            alternate = !alternate;
        }
        return (sum % TEN == 0);
    }

    private String obfuscateNumber(String number) {
        assert number != null;
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < number.length(); i++) {
            if (number.length() > 4 && i >= number.length() - 4) {
                sb.append(number.charAt(i));
            } else {
                sb.append("*");
            }

            if (i < number.length() - 2 && (i + 1) % 4 == 0) {
                sb.append(" ");
            }
        }
        return sb.toString();
    }
}
