/*
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.framework.criteria.jpa;


import java.util.Date;

import javax.persistence.EntityManager;

import dk.apaq.framework.criteria.Criteria;
import dk.apaq.framework.criteria.Rule;
import dk.apaq.framework.criteria.Rules;
import dk.apaq.framework.criteria.Sorter;
import dk.apaq.framework.criteria.Sorter.SorterEntry;
import dk.apaq.framework.criteria.rules.AndRule;
import dk.apaq.framework.criteria.rules.CompareRule;
import dk.apaq.framework.criteria.rules.ContainsRule;
import dk.apaq.framework.criteria.rules.InRule;
import dk.apaq.framework.criteria.rules.IsNullRule;
import dk.apaq.framework.criteria.rules.LikeRule;
import dk.apaq.framework.criteria.rules.NotRule;
import dk.apaq.framework.criteria.rules.OrRule;
import java.util.ArrayList;
import java.util.List;
import junit.framework.TestCase;
import org.junit.Test;

/**
 *
 * @author krog
 */
public class CriteriaForJpaTest extends TestCase {
    
    public CriteriaForJpaTest(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
        super.setUp();
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    EntityManager entityManager = new MockEntityManager();
    Class entityClass = MockBean.class;


    @Test
    public void testIn() {
        List values = new ArrayList();
        values.add("karl");
        values.add("johan");
        
        Rule rule = new InRule("name", values);
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, entityClass, new Criteria(rule));
        assertTrue(result.getStatement().contains("select e from MockBean e WHERE e.name in elements(:param_"));
    }

    @Test
    public void testLike() {
        Rule filter = new LikeRule("name", "collman's*", 0.8F, false);
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, entityClass, new Criteria(filter));
        assertTrue(result.getStatement().contains("select e from MockBean e WHERE UPPER(e.name) LIKE UPPER(:param_"));
    }

     @Test
    public void testLikeCase() {
        Rule filter = new LikeRule("name", "collman's*", 0.8F, true);
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, entityClass, new Criteria(filter));
        assertTrue(result.getStatement().contains("select e from MockBean e WHERE e.name LIKE :param_"));
    }

    @Test
    public void testContains() {
        Rule filter = new ContainsRule("name", "collman's");
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, entityClass, new Criteria(filter));
        assertTrue(result.getStatement().contains(" in elements(e.name)"));
        assertTrue(result.getStatement().contains("select e from MockBean e WHERE :param_"));
    }

    public void testMixed_Date_Compared_IsNull() {
        Rule filter = new OrRule(
                new CompareRule("executeTime", new Date(), CompareRule.CompareType.LessOrEqual),
                new IsNullRule("executeTime"));
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, entityClass, new Criteria(filter));
        assertTrue(result.getStatement().contains("select e from MockBean e WHERE (e.executeTime<=:param_"));
        
    }

    public void testMixed_AND_OR() {
        Rule statusRule = new CompareRule("status", "cool", CompareRule.CompareType.Equals);
        Rule orRule = new OrRule(
                new CompareRule("executeTime", "1233", CompareRule.CompareType.LessOrEqual),
                new IsNullRule("executeTime"));
        Rule andfilter = new AndRule(statusRule, orRule);
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, entityClass, new Criteria(andfilter));
        assertTrue(result.getStatement().contains("select e from MockBean e WHERE (e.status=:param_"));
        assertTrue(result.getStatement().contains("AND (e.executeTime<=:param_"));
        assertTrue(result.getStatement().contains("OR e.executeTime IS NULL"));

    }
    
    public void testMixed_AND_OR2() {
        Rule orRule = new OrRule(
                new CompareRule("executeTime", "1233", CompareRule.CompareType.LessOrEqual),
                new IsNullRule("executeTime"));
        Rule andfilter = new AndRule(orRule);
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, entityClass, new Criteria(andfilter));
        assertTrue(result.getStatement().contains("select e from MockBean e WHERE (e.executeTime<=:param_"));
        assertTrue(result.getStatement().contains("OR e.executeTime IS NULL"));

    }

    public void testNotNull() {
        Rule nullRule = new IsNullRule("name");
        Rule notfilter = new NotRule(nullRule);
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, entityClass, new Criteria(notfilter));
        assertEquals("select e from MockBean e WHERE e.name IS NOT NULL", result.getStatement());

    }

    @Test
    public void testOrderBy() {
        Sorter sorter = new Sorter(new SorterEntry("name", Sorter.Direction.Ascending), new SorterEntry("age", Sorter.Direction.Descending));
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, entityClass, new Criteria(sorter));
        assertEquals(result.getStatement(), "select e from MockBean e ORDER BY e.name ASC, e.age DESC");
    }

    @Test
    public void testFieldsSpecified() {
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, new String[]{"name", "age"}, entityClass, null);
        assertEquals(result.getStatement(), "select e.name, e.age from MockBean e");
    }

    @Test
    public void testCustomRule() {
        CriteriaForJpa.registerInterpreter(MockRule.class, new MockRuleEnterpreter());
        MockRule filter = new MockRule();
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, new String[]{"name", "age"}, entityClass, new Criteria(filter));
        assertEquals(result.getStatement(), "select e.name, e.age from MockBean e WHERE e.mock IS Mocked");
    }
    
    @Test
    public void testEmptyRuleCollections1() {
        Rule rule = Rules.or();
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, new String[]{"name", "age"}, entityClass, new Criteria(rule));
        assertEquals("select e.name, e.age from MockBean e", result.getStatement());
    }
    
    @Test
    public void testEmptyRuleCollections2() {
        Rule rule1 = Rules.or(Rules.like("name", "test"));
        Rule rule2 = Rules.or();
        Rule rule3 = Rules.and(rule1, rule2);
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, new String[]{"name", "age"}, entityClass, new Criteria(rule3));
        assertTrue(result.getStatement().contains("select e.name, e.age from MockBean e WHERE e.name LIKE :param_"));
    }
    
    @Test
    public void testEmptyRuleCollections3() {
        Rule rule1 = Rules.or();
        Rule rule2 = Rules.or();
        Rule rule3 = Rules.and(rule1, rule2);
        MockQuery result = (MockQuery)CriteriaForJpa.translate(entityManager, new String[]{"name", "age"}, entityClass, new Criteria(rule3));
        assertEquals("select e.name, e.age from MockBean e", result.getStatement());
    }

}
