package dk.apaq.framework.criteria.simplequery;

import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;

import dk.apaq.framework.criteria.Rule;
import dk.apaq.framework.criteria.rules.AndRule;
import dk.apaq.framework.criteria.simplequery.parser.QueryParser;
import org.parboiled.Parboiled;
import org.parboiled.errors.ErrorUtils;
import org.parboiled.parserunners.ReportingParseRunner;
import org.parboiled.parserunners.TracingParseRunner;
import org.parboiled.support.ParseTreeUtils;
import org.parboiled.support.ParsingResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A parser for SimpleQuery strings.<br>
 * Takes a SimpleQuery string and parses it to a <code>Rule</code>. The Rule can then be used in a
 * <code>Criteria</code> for resolving data from a data backend.<br>
 * <br>
 * <b>Syntax:</b><br>
 * Equality Operators: = != &lt; &lt;= &gt; &gt;=<br>
 * Logical Operators: AND OR NOT TO TRUE FALSE LIKE CONTAINS IN<br><br>
 * <b>Example:</b><br>
 * (name='karl' AND age=10 AND hairColor!='Brown' AND weight=[55 TO 65] AND favoriteFood IN('chicken','beef') AND computers CONTAINS 'iMac') OR 
 * surname LIKE 'hans*'
 */
public class SimpleQueryParser {

    private static final Logger LOG = LoggerFactory.getLogger(SimpleQueryParser.class);
    private QueryParser compiled;
    private Map<String, String> alias = new LinkedHashMap<String, String>();
    private Map<String, Class> enums = new LinkedHashMap<String, Class>();

    /**
     * Returns aliases.
     */
    public Map<String, String> getAlias() {
        return Collections.unmodifiableMap(alias);
    }

    public Map<java.lang.String, Class> getEnums() {
        return Collections.unmodifiableMap(enums);
    }

    /**
     * Sets aliases
     */
    public SimpleQueryParser setAlias(Map<String, String> alias) {
        this.alias = new LinkedHashMap<String, String>();
        if (alias != null) {
            this.alias.putAll(alias);
        }

        return this;
    }

    public SimpleQueryParser setEnums(Map<String, Class> enums) {
        this.enums = new LinkedHashMap<String, Class>();
        if (enums != null) {
            this.enums.putAll(enums);
        }

        return this;
    }

    public SimpleQueryParser addAlias(String aliasName, String aliasValue) {
        synchronized (alias) {
            alias.put(aliasName, aliasValue);
        }
        return this;
    }

    public SimpleQueryParser addEnum(String propertyName, Class enumValue) {
        if (enumValue == null || !enumValue.isEnum()) {
            throw new IllegalArgumentException("enumValue is null or not an Enum class.");
        }

        synchronized (enumValue) {
            enums.put(propertyName, enumValue);
        }
        return this;
    }

    public SimpleQueryParser clearAlias() {
        this.alias = new LinkedHashMap<String, String>();
        return this;
    }

    public SimpleQueryParser clearEnums() {
        this.enums = new LinkedHashMap<String, Class>();
        return this;
    }

    /**
     * Parse query and use alias added with {@link #addAlia or {@link #setAlia
     */
    public Rule parse(String queryString) {
        if (queryString == null) {
            queryString = "";
        } else {
            queryString = queryString.trim();
        }

        if ("".equals(queryString)) {
            return new AndRule();
        }

        if (alias != null) {
            String queryStringInit = queryString;
            synchronized (alias) {
                // first replace alias in queryString
                for (Map.Entry<String, String> a : alias.entrySet()) {
                    queryString = queryString.replaceAll(a.getKey(), a.getValue());
                }
            }

            if (LOG.isDebugEnabled()) {
                LOG.debug(String.format("QueryString \n'%s' become after alias \n'%s'\naliases are %s",
                        queryStringInit, queryString, alias));
            }
        }

        if (compiled == null) {
            compiled = Parboiled.createParser(QueryParser.class);
        }
        compiled.setEnums(new LinkedHashMap<String, Class>(this.enums));

        ParsingResult<?> result;
        if (LOG.isDebugEnabled()) { 
            result = new TracingParseRunner(compiled.start()).run(queryString);
            LOG.debug("\nParse Tree:\n" + ParseTreeUtils.printNodeTree(result) + '\n');
        } else {
            result = new ReportingParseRunner(compiled.start()).run(queryString);
        }

        if (result.hasErrors() || !result.matched) {
            LOG.debug("\nParse Errors:\n{}", ErrorUtils.printParseErrors(result));
            throw new QueryException(ErrorUtils.printParseErrors(result));
        }

        return (Rule) result.resultValue;

    }
}
