/*
 * Apaq Framework
 * Copyright (C) 2011-2013 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.framework.criteria;

/**
 * Defines a Criteria for resolving data. It contains a
 * <code>Rule</code>,
 * <code>Limit</code> and a
 * <code>Sorter</code> of which all are optional.
 */
public final class Criteria {

    private final Rule rule;
    private final Limit limit;
    private final Sorter sorter;

    /**
     * Creates a Criteria with no Rule, Sorter and Limit.
     */
    public Criteria() {
        this(null, null, null);
    }

    /**
     * Creates a Criteria specifying a Rule.
     *
     * @param rule The rule.
     */
    public Criteria(Rule rule) {
        this(rule, null, null);
    }

    /**
     * Creates a Criteria specifying a Limit.
     *
     * @param limit The limiter
     */
    public Criteria(Limit limit) {
        this(null, null, limit);
    }

    /**
     * Creates a Criteria specifying a Sorter.
     *
     * @param sorter The sorter
     */
    public Criteria(Sorter sorter) {
        this(null, sorter, null);
    }

    /**
     * Creates a Criteria specifying a Rule and a Limit.
     *
     * @param rule The rule.
     * @param limit The limiter
     */
    public Criteria(Rule rule, Limit limit) {
        this(rule, null, limit);
    }

    /**
     * Creates a Criteria specifying a Rule and a Sorter.
     *
     * @param rule The rule
     * @param sorter The limiter
     */
    public Criteria(Rule rule, Sorter sorter) {
        this(rule, sorter, null);
    }

    /**
     * Creates a Criteria specifying a Sorter and a Limiter.
     *
     * @param sorter The sorter
     * @param limit The limiter
     */
    public Criteria(Sorter sorter, Limit limit) {
        this(null, sorter, limit);
    }

    /**
     * Creates a Criteria specifying a Rule, a limiter and a sorter.
     *
     * @param rule The rule
     * @param sorter The sorter
     * @param limit The limiter.
     */
    public Criteria(Rule rule, Sorter sorter, Limit limit) {
        this.rule = rule;
        this.limit = limit;
        this.sorter = sorter;
    }

    /**
     * Retrieves the specified Limit.
     *
     * @return The Limit or null if Criteria specifies not Limit.
     */
    public Limit getLimit() {
        return limit;
    }

    /**
     * Retrieves the specified Rule.
     *
     * @return The Rule or null if Criteria specifies not Rule.
     */
    public Rule getRule() {
        return rule;
    }

    /**
     * Retrieves the specified Sorter.
     *
     * @return The Sorter or null if Criteria specifies not Sorter.
     */
    public Sorter getSorter() {
        return sorter;
    }
}
