/*
 * Apaq Framework
 * Copyright (C) 2011-2013 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.framework.criteria;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import dk.apaq.framework.criteria.rules.SimpleFiltrationItem;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 */
public final class CriteriaUtil {

    private static final Pattern BEANPATTERN_GET = Pattern.compile("get[A-Z][a-zA-Z0-9]*");
    private static final Pattern BEANPATTERN_IS = Pattern.compile("is[A-Z][a-zA-Z0-9]*");
    private static final Logger LOG = LoggerFactory.getLogger(CriteriaUtil.class);

    private CriteriaUtil() { /* EMPTY */ }

    private static List limitList(List list, Limit limit) {
        if (limit != null) {
            int start = Math.min(limit.getOffset(), list.size());
            int end = Math.min(limit.getOffset() + limit.getCount(), list.size());
            list = list.subList(start, end);
        }
        return list;
    }

    private static <T> List<T> filterCollectionWithoutRuleOrSorter(Collection<T> c, Criteria criteria) {
        List<T> result = new ArrayList<T>(c);
        result = limitList(result, criteria.getLimit());
        return result;
    }

    private static <T> List<T> filterCollectionWithRuleOrSorter(Collection<T> c, Criteria criteria) {
        Map<Rule.FiltrationItem, T> map = new HashMap<Rule.FiltrationItem, T>();
        List<Rule.FiltrationItem> intermediates = new ArrayList<Rule.FiltrationItem>();
        for (T element : c) {
            Rule.FiltrationItem fi = buildItemFromBean(element);
            if (criteria.getRule() == null || criteria.getRule().passesRule(fi)) {
                intermediates.add(fi);
                map.put(fi, element);
            }
        }
        if (criteria.getSorter() != null) {
            Collections.sort(intermediates, new Sorter.Comparator(criteria.getSorter()));
        }
        List<T> result = new ArrayList<T>();

        intermediates = limitList(intermediates, criteria.getLimit());

        for (Rule.FiltrationItem item : intermediates) {
            result.add(map.get(item));
        }
        return result;
    }
    
    /**
     * Builds a FiltrationImte from a Bean.
     * @param object The bean to convert.
     * @return The generated FiltrationItem.
     */
    public static Rule.FiltrationItem buildItemFromBean(Object object) {
        SimpleFiltrationItem item = new SimpleFiltrationItem();

        Method[] methods = object.getClass().getMethods();
        for (Method method : methods) {

            boolean prefixedWithGet = BEANPATTERN_GET.matcher(method.getName()).matches();
            boolean prefixedWithIs = BEANPATTERN_IS.matcher(method.getName()).matches();
            if (method.getParameterTypes().length == 0
                    && method.getReturnType() != Void.class
                    && (prefixedWithGet || prefixedWithIs)) {
                try {
                    Object returnVal = method.invoke(object);
                    int cutIndex = prefixedWithIs ? "is".length() : "get".length();
                    String propertyName = method.getName().substring(cutIndex, cutIndex + 1).toLowerCase() + method.getName().substring(cutIndex + 1);
                    item.setProperty(propertyName, returnVal);
                } catch (Exception ex) {
                    LOG.error("Error while building filtrationitem from bean.", ex);
                }
            }
        }
        return item;
    }

    /**
     * Filters a Collections of Beans based on the given Rule.
     * @param <T> The bean type
     * @param c The collection of Beans.
     * @param rule The Rule used for filtering.
     * @return A List of beans from the given Collection filtered according to the given Rule.
     */
    public static <T> List<T> filterCollection(Collection<T> c, Rule rule) {
        return filterCollection(c, new Criteria(rule));
    }

    /**
     * Filters a Collections of Beans based on the given Criteria.
     * @param <T> The bean type
     * @param c The collection of Beans.
     * @param criteria The Criteria used for filtering.
     * @return A List of beans from the given Collection filtered according to the given Criteria.
     */
    public static <T> List<T> filterCollection(Collection<T> c, Criteria criteria) {
        criteria = criteria == null ? new Criteria() : criteria;

        if (criteria.getRule() == null && criteria.getSorter() == null) {
            return filterCollectionWithoutRuleOrSorter(c, criteria);
        } else {
            return filterCollectionWithRuleOrSorter(c, criteria);
        }
    }
}
