package dk.apaq.framework.repository.jpa;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Query;
import dk.apaq.framework.criteria.Criteria;
import dk.apaq.framework.criteria.Limit;
import dk.apaq.framework.criteria.jpa.CriteriaForJpa;
import dk.apaq.framework.repository.Repository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.jpa.repository.support.JpaEntityInformation;
import org.springframework.data.jpa.repository.support.JpaEntityInformationSupport;
import org.springframework.data.jpa.repository.support.SimpleJpaRepository;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 */
@Transactional(readOnly = true)
public class EntityManagerRepository<BEANTYPE, IDTYPE extends Serializable> 
    extends SimpleJpaRepository<BEANTYPE, IDTYPE> 
    implements Repository<BEANTYPE, IDTYPE> {

    private static final Logger LOG = LoggerFactory.getLogger(EntityManagerRepository.class);
    private final EntityManager entityManager;
    private final Class<BEANTYPE> clazz;
    private JpaEntityInformation<BEANTYPE, IDTYPE> entityInformation;
    private boolean transactionsHandled = false;
    
    public EntityManagerRepository(EntityManager entityManager, Class<BEANTYPE> clazz) {
        super(clazz, entityManager);
        this.entityManager = entityManager;
        this.clazz = clazz;
        entityInformation = (JpaEntityInformation<BEANTYPE, IDTYPE>) JpaEntityInformationSupport.getMetadata(clazz, entityManager);

        if (entityManager == null) {
            throw new NullPointerException("EntityManager was null");
        }
        if (clazz == null) {
            throw new NullPointerException("Class was null");
        }

    }

    public long count(Criteria criteria) {
        Query q = CriteriaForJpa.translate(entityManager, null, clazz, criteria, true);
        return (Long)q.getSingleResult();
    }
    
    public List<BEANTYPE> findAll(Criteria criteria) {
        Query q = CriteriaForJpa.translate(entityManager, clazz, criteria);
        return q.getResultList();
    }


    public BEANTYPE findOne(Criteria criteria) {
        criteria = criteria == null ? new Criteria() : criteria;
        Limit limit = criteria.getLimit() == null ? new Limit(0, 1) : new Limit(criteria.getLimit().getOffset(), 1);
        criteria = new Criteria(criteria.getRule(), criteria.getSorter(), limit);
        
        List<BEANTYPE> result = findAll(criteria);
        return result.isEmpty() ? null : result.get(0);
    }

    @Override
    @Transactional
    public void delete(final Iterable<? extends BEANTYPE> entities) {
        runInTransaction(new Runnable() {
            public void run() {
                EntityManagerRepository.super.delete(entities);
            }
        });
    }

    @Override
    @Transactional
    public void deleteAll() {
        runInTransaction(new Runnable() {
            public void run() {
                EntityManagerRepository.super.deleteAll();
            }
        });
    }

    @Override
    @Transactional
    public void deleteAllInBatch() {
        runInTransaction(new Runnable() {
            public void run() {
                EntityManagerRepository.super.deleteAllInBatch();
            }
        });
    }

    @Override
    @Transactional
    public void deleteInBatch(final Iterable<BEANTYPE> entities) {
        runInTransaction(new Runnable() {
            public void run() {
                EntityManagerRepository.super.deleteInBatch(entities);
            }
        });
    }


    @Override
    @Transactional
    public void delete(final BEANTYPE entity) {
        IDTYPE id = entityInformation.getId(entity);
        runInTransaction(new Runnable() {
            public void run() {
                EntityManagerRepository.super.delete(entity);
            }
        });
    }

    @Override
    @Transactional
    public void delete(final IDTYPE id) {
        runInTransaction(new Runnable() {
            public void run() {
                EntityManagerRepository.super.delete(id);
            }
        });
    }
    
    @Override
    @Transactional
    public <S extends BEANTYPE> S save(final S entity) {
        final Object[] entityA = new Object[1];
        runInTransaction(new Runnable() {
            public void run() {
                entityA[0] = EntityManagerRepository.super.save(entity);
            }
        });
        return (S) entityA[0];
    }

    /**
     * Specifies whether the entity provider should handle transactions itself or whether they should be handled outside (e.g. if declarative
     * transactions are used).
     *
     * @param transactionsHandled true to handle the transactions internally, false to rely on external transaction handling.
     */
    public void setTransactionsHandled(boolean transactionsHandled) {
        this.transactionsHandled = transactionsHandled;
    }

    /**
     * Returns whether the entity provider is handling transactions internally (the default) or relies on external transaction handling.
     *
     * @return true if transactions are handled internally, false if not.
     */
    public boolean isTransactionsHandled() {
        return transactionsHandled;
    }

    /**
     * If {@link #isTransactionsHandled() } is true,
     * <code>operation</code> will be executed inside a transaction that is commited after the operation is completed. Otherwise,
     * <code>operation</code> will just be executed.
     *
     * @param operation the operation to run (must not be null).
     */
    protected void runInTransaction(Runnable operation) {
        assert operation != null : "operation must not be null";
        if (isTransactionsHandled()) {
            EntityTransaction et = entityManager.getTransaction();
            if (et.isActive()) {
                // The transaction has been started outside of this method
                // and should also be committed/rolled back outside of
                // this method
                operation.run();
            } else {
                try {
                    et.begin();
                    operation.run();
                    et.commit();
                } finally {
                    if (et.isActive()) {
                        et.rollback();
                    }
                }
            }
        } else {
            operation.run();
        }
    }
    
}
