package dk.apaq.framework.repository.rest;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import dk.apaq.framework.criteria.Criteria;
import dk.apaq.framework.criteria.Limit;
import dk.apaq.framework.criteria.simplequery.CriteriaForSimpleQuery;
import dk.apaq.framework.repository.Repository;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageConversionException;
import org.springframework.web.client.HttpStatusCodeException;
import org.springframework.web.client.RestOperations;

/**
 * Javadoc
 */
public class RestRepository<BT> implements Repository<BT, String> {

    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(RestRepository.class);
    private final String repositoryUrl;
    private final RestOperations restOp;
    private final Class<BT> clazz;
    private String countParameter = "count";
    private String offsetParameter = "offset";
    private String queryParameter = "query";
    private String encoding = "utf-8";


    public RestRepository(String repositoryUrl, RestOperations restOp, Class clazz) {
        this.repositoryUrl = repositoryUrl;
        this.restOp = restOp;
        this.clazz = clazz;
    }

    public String getIdFromEntity(BT entity) {
        try {
            Field field = entity.getClass().getDeclaredField("id");
            field.setAccessible(true);
            return (String) field.get(entity);
        } catch (Exception ex) {
            LOG.error("Unable to retrieve id for entity[class={}]", clazz);
        }
        return null;
    }

    @Override
    public BT findOne(String id) {
        LOG.debug("Retrieving one entity. [class={}; id={}]", clazz, id);
        BT entity = null;
        try {
            entity = restOp.getForObject(repositoryUrl + "/" + id, clazz);
        } catch (HttpStatusCodeException ex) {
            if (ex.getStatusCode() != HttpStatus.NOT_FOUND) {
                LOG.error("Unexpected HttpStatus when requesting data.", ex);
                throw ex;
            }
        } catch (HttpMessageConversionException ex) {
            LOG.error("Unreadable message when requesting data.", ex);
            throw ex;
        }
        return entity;
    }

    @Override
    public List<BT> findAll(Criteria criteria) {
        
        /*TODO 
         * Add support for retrieving a full list of entity objects
         * on rest services that supports it. It will be much faster than
         * first retreiving a list of id's and then retrieving one at the time. 
         */
        
        LOG.debug("Retrieving all matching criteria. [class={}; criteria={}]", clazz, criteria);
        List<String> idlist = findAllIds(criteria);
        List<BT> result = new ArrayList<BT>();
        
        for (String id : idlist) {
            result.add(findOne(id));
        }
        return result;
    }

    public long count(Criteria criteria) {
        return findAllIds(new Criteria(criteria.getRule(), new Limit(Integer.MAX_VALUE))).size();
    }

    
    public List<String> findAllIds(Criteria criteria) {
        LOG.debug("Retrieving all ids matching criteria. [class={}; criteria={}]", clazz, criteria);
        Map<String, String> variables = new HashMap<String, String>();
        StringBuilder queryString = new StringBuilder();

        if (criteria != null) {
            if (criteria.getLimit() != null) {
                appendQueryParameter(queryString, offsetParameter);
                variables.put(offsetParameter, Integer.toString(criteria.getLimit().getOffset()));
                appendQueryParameter(queryString, countParameter);
                variables.put(countParameter, Integer.toString(criteria.getLimit().getCount()));
                
            }

            if (criteria.getRule() != null) {
                appendQueryParameter(queryString, queryParameter);
                variables.put(queryParameter, CriteriaForSimpleQuery.translate(criteria.getRule()));
            }
        }
        
        
        String expandedUrl = repositoryUrl + queryString.toString();
        LOG.debug("Requesting data: {}", expandedUrl);
        
        return restOp.getForObject(expandedUrl, List.class, variables);
    }

    public void delete(String id) {
        LOG.debug("Deleting entity [class={}; id={}]", clazz, id);
        restOp.delete(repositoryUrl + "/" + id);
    }

    @Override
    public void delete(BT entity) {
        delete(getIdFromEntity(entity));
    }

    public <S extends BT> S save(S entity) {
        String id = getIdFromEntity(entity);
        BT result = null;
        if (id == null) {
            LOG.debug("Persisting new entity. [class={}]", clazz);
            HttpEntity<BT> httpEntity = new HttpEntity<BT>(entity);
            ResponseEntity<String> responseEntity = restOp.exchange(repositoryUrl, HttpMethod.PUT, httpEntity, String.class);
            id = responseEntity.getBody();
            result = findOne(id);
        } else {
            LOG.debug("Updating entity. [class={};]", clazz);
            result = restOp.postForObject(repositoryUrl + "/" + id, entity, clazz);
        }
        return (S) result;
    }
    
    public void setCountParameter(String countParameter) {
        this.countParameter = countParameter;
    }

    public void setOffsetParameter(String offsetParameter) {
        this.offsetParameter = offsetParameter;
    }

    public void setQueryParameter(String queryParameter) {
        this.queryParameter = queryParameter;
    }

    public void setEncoding(String encoding) {
        this.encoding = encoding;
    }

    private void appendQueryParameter(StringBuilder sb, String paramName) {
        if (sb.length() == 0) {
            sb.append("?");
        } else {
            sb.append("&");
        }
        
        sb.append(paramName).append("={").append(paramName).append("}");
        
    }


    public <S extends BT> Iterable<S> save(Iterable<S> entities) {
        List<S> result = new ArrayList<S>();
        for(S e : entities) {
            result.add(save(e));
        }
        return result;
    }

    public boolean exists(String id) {
        try {
            restOp.headForHeaders(repositoryUrl + "/" + id);
            return true;
        } catch (HttpStatusCodeException ex) {
            if (ex.getStatusCode() != HttpStatus.NOT_FOUND) {
                LOG.error("Unexpected HttpStatus when requesting data.", ex);
                throw ex;
            }
            return false;
        }
    }

    public Iterable<BT> findAll() {
        return findAll((Criteria)null);
    }

    public Iterable<BT> findAll(Iterable<String> ids) {
        return findAll((Criteria)null);
    }

    public long count() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void delete(Iterable<? extends BT> entities) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void deleteAll() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public BT findOne(Criteria criteria) {
        Iterable<BT> result = findAll(new Criteria(criteria.getRule(), criteria.getSorter(), new Limit(1)));
        Iterator<BT> it = result.iterator();
        if(it.hasNext()) {
            return it.next();
        } else {
            return null;
        }
        
    }

}
