package dk.apaq.framework.repository;

import java.io.Serializable;
import java.util.*;
import dk.apaq.framework.criteria.Criteria;
import dk.apaq.framework.criteria.CriteriaUtil;
import dk.apaq.framework.criteria.Limit;
import dk.apaq.framework.criteria.Rule;
import dk.apaq.framework.criteria.Sorter;

/**
 *
 */
public class CollectionRepository<BEANTYPE> implements Repository<BEANTYPE, String> {

    private final List<BEANTYPE> beans;
    private final IdResolver<BEANTYPE> idResolver;

    private static class DefaultIdResolver<BEANTYPE> implements IdResolver<BEANTYPE> {

        private final WeakHashMap<BEANTYPE, String> idmap = new WeakHashMap<BEANTYPE, String>();

        public String getIdForBean(BEANTYPE bean) {
            String id = idmap.get(bean);
            if (id == null) {
                id = UUID.randomUUID().toString();
                idmap.put(bean, id);
            }
            return id;
        }
    }

    public interface IdResolver<BEANTYPE> {

        String getIdForBean(BEANTYPE bean);
    }

    public CollectionRepository() {
        this(null, new DefaultIdResolver<BEANTYPE>());
    }

    public CollectionRepository(IdResolver<BEANTYPE> idResolver) {
        this(null, idResolver);
    }

    public CollectionRepository(List<BEANTYPE> beans) {
        this(beans, new DefaultIdResolver<BEANTYPE>());
    }

    public CollectionRepository(List<BEANTYPE> beans, IdResolver<BEANTYPE> idResolver) {
        this.beans = beans == null ? new ArrayList<BEANTYPE>() : beans;
        this.idResolver = idResolver;
    }

    public List<BEANTYPE> getBeans() {
        return beans;
    }

    public BEANTYPE findOne(Criteria criteria) {
        criteria = criteria == null ? new Criteria() : criteria;
        Limit limit = criteria.getLimit() == null ? new Limit(0, 1) : new Limit(criteria.getLimit().getOffset(), 1);
        criteria = new Criteria(criteria.getRule(), criteria.getSorter(), limit);
        
        List<BEANTYPE> result = findAll(criteria);
        return result.isEmpty() ? null : result.get(0);
    }

    public Iterable<BEANTYPE> findAll() {
        return findAll((Criteria)null);
    }

    public String getIdFromEntity(BEANTYPE entity) {
        return idResolver.getIdForBean(entity);
    }

    public <S extends BEANTYPE> Iterable<S> save(Iterable<S> entities) {
        List<S> result = new ArrayList<S>();
        
        for(S entity : entities) {
            result.add(save(entity));
        }
        return Collections.unmodifiableList(result);
    }

    public <S extends BEANTYPE> S save(S entity) {
        beans.add(entity);
        return entity;
    }

    public long count() {
        return beans.size();
    }
    
    public long count(Criteria criteria) {
        return findAll(criteria).size();
    }
    

    public void delete(Iterable<? extends BEANTYPE> entities) {
        for(BEANTYPE entity : entities) {
            delete(entity);
        }
    }

    public void deleteAll() {
        beans.clear();
    }
    
    public void delete(BEANTYPE entity) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void delete(String id) {
        Iterator<BEANTYPE> it = beans.iterator();

        while (it.hasNext()) {
            BEANTYPE b = it.next();
            if (id.equals(idResolver.getIdForBean(b))) {
                it.remove();
                break;
            }
        }
    }

    public BEANTYPE findOne(String id) {
        for (BEANTYPE current : beans) {
            String currentId = idResolver.getIdForBean(current);
            if (id.equals(currentId)) {
                return current;
            }
        }
        return null;
    }
    
    public Iterable<BEANTYPE> findAll(Iterable<String> ids) {
        List<BEANTYPE> result = new ArrayList<BEANTYPE>();
        for(String id : ids) {
            result.add(findOne(id));
        }
        return Collections.unmodifiableList(result);
    }

    public List<BEANTYPE> findAll(Criteria criteria) {
        List<String> idlist = findAllIds(criteria);
        List<BEANTYPE> beanlist = new ArrayList<BEANTYPE>();

        for (String id : idlist) {
            beanlist.add(findOne(id));
        }
        return beanlist;

    }

    public List<String> findAllIds(Criteria criteria) {
        Rule rule = criteria == null ? null : criteria.getRule();
        Limit limit = criteria == null ? null : criteria.getLimit();
        Sorter sorter = criteria == null ? null : criteria.getSorter();

        List<BEANTYPE> list = CriteriaUtil.filterCollection(beans, criteria);
        List<String> idlist = new ArrayList<String>();
        for (BEANTYPE e : list) {
            String id = idResolver.getIdForBean(e);
            idlist.add(id);
        }

        return idlist;
    }

    public boolean exists(String id) {
        if (id != null) {
            for (BEANTYPE entity : beans) {
                if (id.equals(getIdFromEntity(entity))) {
                    return true;
                }
            }
        }
        return false;
    }
}
