package dk.apaq.framework.web.filters;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A filter for handling host alias redirection. <br><br>
 * 
 * Each host can have multiple aliases and have each request to an
 * alias redirected to the real host.<br><br>
 * 
 * <b>Example for http://simplepay.dk:</b><br>
 * Host: simplepay.dk<br>
 * Aliases: www.simplepay.dk, web.simplepay.dk<br>
 * <br>
 * When a browser requests www.simplepay.dk or web.simplepay.dk it will be 
 * redirected to simplepay.dk with a HTTP 301.<br>
 * <br>
 * <b>Configuration:</b><br>
 * The filter takes one argument: hosts<br>
 * This argument must use the following syntax.:<br>
 * host,alias[,alias][;host,alias[,alias]]<br>
 * Example: simplepay.dk,www.simplepay.dk,web.simplepay.dk;ahost.com,www.ahost.com
 */
public class AliasFilter implements Filter {

    private List<Host> hosts = new ArrayList<Host>();
    
    private class Host {
        private String name;
        private List<String> aliases = new ArrayList<String>();

        public Host(String name, String ... aliases) {
            this.name = name;
            this.aliases = new ArrayList<String>(Arrays.asList(aliases));
        }
        
        public Host(String name, List<String> aliases) {
            this.name = name;
            this.aliases = aliases;
        }

        public String getName() {
            return name;
        }

        public List<String> getAliases() {
            return aliases;
        }
        
    }
    
    public void init(FilterConfig fc) throws ServletException {
        String strHosts = fc.getInitParameter("hosts");
        if(strHosts != null) {
            String[] hostsArray = strHosts.split(";");
            for(String hostString : hostsArray) {
                String[] hostData = hostString.split(",");
                hosts.add(new Host(hostData[0], Arrays.copyOfRange(hostData, 1, hostData.length)));
            }
        }
    }

    public void doFilter(ServletRequest req, ServletResponse resp, FilterChain chain) throws IOException, ServletException {
        if(req instanceof HttpServletRequest) {
            HttpServletRequest request = (HttpServletRequest) req;
            HttpServletResponse response = (HttpServletResponse) resp;
            URL url = new URL(request.getRequestURL().toString());
            
            for(Host host : hosts) {
                
                //if host matches the url's host name, then just break the loop
                if(host.getName().equals(url.getHost())) break;
                
                //Loop through aliases to test for a match
                for(String alias : host.getAliases()) {
                    if(alias.equals(url.getHost())) {
                        response.sendRedirect(generateRedirectUrl(url, host.getName()));
                        return;
                    }
                }
            }
        }
        
        //There were no reason to redirect. Carry on
        chain.doFilter(req, resp);
    }

    public void destroy() { /* EMPTY */ }
    
    private String generateRedirectUrl(URL url, String newHost) {
        boolean addPort = ("http".equals(url.getProtocol()) && url.getPort() != 80) ||
                          ("https".equals(url.getProtocol()) && url.getPort() != 443);
        
        StringBuilder sb = new StringBuilder();
        sb.append(url.getProtocol());
        sb.append("://");
        
        if(url.getUserInfo() != null) {
            sb.append(url.getUserInfo());
            sb.append("@");
        }
        
        sb.append(newHost);
        
        if(addPort) {
            sb.append(":");
            sb.append(url.getPort());
        }
        
        sb.append(url.getFile());
        
        if(url.getRef() != null) {
            sb.append("#");
            sb.append(url.getRef());
        }
        
        return sb.toString();
    }
    
}
