package dk.apaq.framework.web.filters;

import java.io.IOException;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Filter for Cross-Origin Resource Sharing(CORS).<br>
 * <br>
 * Allows configuration of the following headers:<br><br>
 * 
 * <b>Access-Control-Allow-Origin</b><br>
 * This header can be configured by setting 'allowedOrigins' with a
 * commaseperated list of origins. By default all origins are allowed.<br><br>
 * 
 * <b>Access-Control-Allow-Headers</b><br>
 * This header can be configured by setting 'allowedHeaders' with a string specifying
 * the exact header content.<br><br>
 * 
 * <b>Access-Control-Allow-Methods</b><br>
 * This header can be configured by setting 'allowedMethods' with a string specifying
 * the exact header content.
 * 
 */
public class Corsfilter implements Filter {
    
    private String allowedHeaders = null;
    private String allowedMethods = "GET,POST,OPTIONS";
    private String[] allowedOrigins = null;
    
    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        
        if(response instanceof HttpServletResponse) {
            HttpServletRequest httpReq = (HttpServletRequest) request;
            HttpServletResponse httpRes = (HttpServletResponse) response;
            
            boolean preflight = isPreflight(httpReq);
            String origin = httpReq.getHeader("Origin");
            
            if(allowedOrigins == null) {
                httpRes.setHeader("Access-Control-Allow-Origin", origin);
            } else {
                for(String allowedOrigin : allowedOrigins) {
                    if(allowedOrigin.equals(origin)) {
                        httpRes.setHeader("Access-Control-Allow-Origin", origin);
                        break;
                    }
                }
            }
            
            httpRes.setHeader("Access-Control-Allow-Credentials", "true");
            
            if(allowedHeaders != null) {
                httpRes.setHeader("Access-Control-Allow-Headers", allowedHeaders);
            }
            
            httpRes.setHeader("Access-Control-Allow-Methods", allowedMethods);
            
            if(preflight) return;
            
            chain.doFilter(httpReq, httpRes);
        } else {
            chain.doFilter(request, response);
        }
    }
    
    private boolean isPreflight(HttpServletRequest request) {
        return request.getHeader("Access-Control-Request-Method") != null;
    }
    
    /**
     * Init method for this filter 
     */
    @Override
    public void init(FilterConfig filterConfig) { 
        allowedHeaders = filterConfig.getInitParameter("allowedHeaders");
        
        String strMethods = filterConfig.getInitParameter("allowedMethods");
        if(strMethods != null) {
            allowedMethods = strMethods;
        }
        
        String strOrigins = filterConfig.getInitParameter("allowedOrigins");
        if(strOrigins != null) {
            allowedOrigins = strOrigins.split(",");
        }
    }

    @Override
    public void destroy() { }

}
