package dk.apaq.framework.web.filters;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.ArrayList;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletResponse;

/**
 * A filter that creates virtual subpages for each supported locales. This makes it possible to
 * let a multilanguage site have each translation of the site on seperate unique URL's.<br><br>
 * 
 * It works in the following way. If defaultLanguage is 'en' and the list of supported languages
 * are en, da and fo then the following urls will all direct to the same servlet handling:<br>
 * - http://host/ (English locale - 'en') - Default<br>
 * - http://host/da (Danish locale - 'da')<br>
 * - http://host/fo (Faroese locale - 'fo')<br>
 * <br><br>
 * The filter automatically redirects to the locale specified by the users browsers. If the locale
 * is not among the locale specified as supported by the filter, then the default locale is chosen.
 * For example, a user from Denmark('da') requesting http://host/ would be redirected to http://host/da
 * if 'da' is a supported locale. Otherwise his locale would be changed to the default locale(fx. 'en').<br><br>
 * 
 * The filter also respects forcing a specific locale. This shows when fx. a user from Denmark request
 * a faroese translation(http://host/fo). In this case the filter will not redirect him. It will instead
 * change his locale to the Faroese locale('fo').<br><br>
 * 
 * <b>Configuration</b><br>
 * The filter supports the following configuration:<br>
 * - defaultLanguage (fx. 'en')<br>
 * - supportedLanguages (commaseperated, fx. 'en,da,fo')<br>
 * - ignoredPaths (commaseperated list of paths which the filter should not handle, fx. '/da,/sv')<br>
 * 
 */
public class LocaleFilter  implements Filter {
    private String defaultLanguage = "en";
    private final List<String> supportedLanguages = new ArrayList(Arrays.asList(new String[]{"en"}));
    private final List<String> ignoredPaths = new ArrayList<String>();

    private class LocaleWrappingRequest extends HttpServletRequestWrapper {

        private final Locale locale;

        public LocaleWrappingRequest(HttpServletRequest request, Locale locale) {
            super(request);
            this.locale = locale; 
        }

        @Override
        public Locale getLocale() {
            return locale;
        }
        
    }

    public void destroy() { /* EMPTY */ }
    
    @Override
    public void doFilter(ServletRequest request, ServletResponse response,
            FilterChain chain) throws IOException, ServletException {
        
        if (request instanceof HttpServletRequest && !Boolean.TRUE.equals(request.getAttribute("LOCALE_SET"))) {
            HttpServletRequest httpRequest = (HttpServletRequest) request;
            HttpServletResponse httpResponse = (HttpServletResponse) response;

            String path = httpRequest.getServletPath();

            if (!isPathIgnored(path)) {

                if (httpRequest.getQueryString() != null) {
                    path += "?" + httpRequest.getQueryString();
                }

                boolean localeInPath = false;
                
                Locale locale = getLocaleFromPath(path);
                if (locale == null) {
                    //Url has no indication of Locale - Get browsers locale
                    Locale userLocale = request.getLocale();

                    //if browsers locale is not default language but is a supported
                    //language, then redirect to that language
                    if (!defaultLanguage.equals(userLocale.getLanguage())
                            && isLanguageSupported(userLocale)) {
                        httpResponse.sendRedirect("/" + userLocale.getLanguage() + path);
                        return;
                    }
                    
                    
                } else {
                    localeInPath = true;
                }

                if (locale == null) {
                    locale = Locale.ENGLISH;
                }
                
                if(localeInPath) {
                    request = new LocaleWrappingRequest(httpRequest, locale);
                    request.setAttribute("LOCALE_SET", true);
                    RequestDispatcher dispatcher = httpRequest.getRequestDispatcher(path.substring(3));
                    dispatcher.forward(request, response);
                    return;
                }

            }
        }

        chain.doFilter(request, response);

    }

    public void init(FilterConfig filterConfig) throws ServletException {
        String ignored = filterConfig.getInitParameter("ignoredPaths");
        if(ignored!=null) {
            ignoredPaths.addAll(Arrays.asList(ignored.split(",")));
        }
        
        String langs = filterConfig.getInitParameter("supportedLanguages");
        if(langs!=null) {
            supportedLanguages.clear();
            supportedLanguages.addAll(Arrays.asList(langs.split(",")));
        }
        
        String def = filterConfig.getInitParameter("defaultLanguage");
        if(def!=null) {
            this.defaultLanguage = def;
        }
    }

    private Locale getLocaleFromPath(String path) {
        //The context of the users request revolve his wanted locale
        //if first path element if two-lettered and one of the supported locales
        //then we resolve the locale, remove the context and set the locale as property
        //if no matching path element is found, then default to english.
        String lang = "en";
        Locale locale = null;

        if (path != null && path.length() > 2) {
            String tmp = path;

            //It should according to standard start with a slash
            //We will remove it before splitting to array
            tmp = tmp.substring(1);

            String[] array = tmp.split("/");
            if (array[0].length() == 2) {
                lang = array[0];

                locale = getLocaleFromLanguageCode(lang);
            }

        }
        return locale;
    }

    /*private Locale getLocaleFromCountryCode(String country) {
        for (Locale locale : Locale.getAvailableLocales()) {
            if (locale.getCountry().equals(country)) {
                return locale;
            }
        }
        return null;
    }*/

    private Locale getLocaleFromLanguageCode(String language) {
        for (Locale locale : Locale.getAvailableLocales()) {
            if (locale.getLanguage().equals(language)) {
                return locale;
            }
        }
        return null;
    }

    private boolean isLanguageSupported(Locale locale) {
        if (locale == null) {
            return false;
        }
        for (String language : supportedLanguages) {
            if (language.equals(locale.getLanguage())) {
                return true;
            }
        }
        return false;
    }

    private boolean isPathIgnored(String path) {
        for(String current : ignoredPaths) {
            if(path.startsWith(current)) {
                return true;
            }
        }
        return false;
    }

}
